from .orm import Engine
from .config import POETRY_TABLE, DATABASE


class Poetry:
    def __init__(self):
        self.engine = Engine(DATABASE)
        # 题目，作者名，朝代，类型，内容
        self.field = ('name', 'author', 'dynasty', 'type', 'content')

    def _tuple2dict(self, queryset):
        """
        :param queryset: 查询结果集
        :type queryset: list[tuple]
        :return: 转换为列表套字典的结果集
        :rtype: list[dict[str, str]]
        """
        return [dict(zip(self.field, row)) for _, *row in queryset]

    def for_author(self, a, method=0):
        """
        :param a: 作者名
        :type a: str
        :param method: 匹配模式（0为精准匹配，1为相似匹配）
        :type method: int
        :return: 多个匹配行的列表
        :rtype: list[dict[str, str]]
        """
        return self._tuple2dict(self.engine.filter(POETRY_TABLE, method, author=a))

    def for_type(self, t):
        """
        :param t: 类型（诗 词 曲 文言文）
        :type t: str
        :return: 多个匹配行的列表
        :rtype: list[dict[str, str]]
        :raise method: 精准匹配
        """
        return self._tuple2dict(self.engine.filter(POETRY_TABLE, 0, type=t))

    def for_dynasty(self, d, method=0):
        """
        :param d: 朝代（先秦 唐代 元代 五代 两汉 魏晋 南北朝 宋代 明代 清代 金朝 近现代 未知）
        :type d: str
        :param method: 匹配模式（0为精准匹配，1为相似匹配）
        :type method: int
        :return: 多个匹配行的列表
        :rtype: list[dict[str, str]]
        """
        return self._tuple2dict(self.engine.filter(POETRY_TABLE, method, dynasty=d))

    def for_name(self, n, method=0):
        """
        :param n: 题目
        :type n: str
        :param method: 匹配模式（0为精准匹配，1为相似匹配）
        :type method: int
        :return: 多个匹配行的列表
        :rtype: list[dict[str, str]]
        """
        return self._tuple2dict(self.engine.filter(POETRY_TABLE, method, name=n))

    def for_content(self, c):
        """
        :param c: 内容
        :type c: str
        :return: 多个匹配行的列表
        :rtype: list[dict[str, str]]
        :raise method: 相似匹配
        """
        return self._tuple2dict(self.engine.filter(POETRY_TABLE, method=1, content=c))

    def for_customize(self, method=0, **kwargs):
        """
        :param method: 匹配模式（0为精准匹配，1为相似匹配）
        :type method: int
        :keyword name: 题目
        :keyword author: 作者名
        :keyword dynasty: 朝代（先秦 唐代 元代 五代 两汉 魏晋 南北朝 宋代 明代 清代 金朝 近现代 未知）
        :keyword type: 类型（诗 词 曲 文言文）
        :keyword content: 内容
        :return: 多个匹配行的列表
        :rtype: list[dict[str, str]]

        自定义匹配
        """
        return self._tuple2dict(self.engine.filter(POETRY_TABLE, method=method, **kwargs))
